function [eta, eta_max] = estimate_error(T, u, f)
    % Estimate error using residual-based method
    n = length(T) - 1;
    eta = zeros(n, 1);
    for k = 1:n
        h = T(k+1) - T(k);
        uh_prime = (u(k+1) - u(k)) / h;
        if k > 1
            uh_prime_l = uh_prime - (u(k) - u(k-1))/(T(k) - T(k-1));
        else
            uh_prime_l = 0;
        end
        if k < n
            uh_prime_r = (u(k+2) - u(k+1))/(T(k+2) - T(k+1)) - uh_prime;
        else
            uh_prime_r = 0;
        end
        residual = @(x) f(x);
        eta(k) = h.^2 * integral(@(x) residual(x).^2, T(k), T(k+1)) + 0.5*(uh_prime_l^2 + uh_prime_r^2);
    end
    eta_max = max(sqrt(eta));
    eta = sqrt(eta);
end