function [u, K_global] = solve_problem(T, f)
    % Solve the discrete problem using FEM
    n = length(T) - 1;
    K_global = zeros(n+1, n+1);
    F_global = zeros(n+1, 1);
    
    % Assemble global stiffness matrix and load vector
    for k = 1:n
        h = T(k+1) - T(k);
        K_local = [1, -1; -1, 1] / h;
        F_local = [integral(@(x) f(x).*(1 - (x-T(k))/h), T(k), T(k+1)), ...
                   integral(@(x) f(x).*((x-T(k))/h), T(k), T(k+1))]';
        K_global(k:k+1, k:k+1) = K_global(k:k+1, k:k+1) + K_local;
        F_global(k:k+1) = F_global(k:k+1) + F_local;
    end
    
    % Apply Dirichlet boundary conditions
    K_global(1, :) = 0; K_global(:, 1) = 0; K_global(1, 1) = 1;
    K_global(end, :) = 0; K_global(:, end) = 0; K_global(end, end) = 1;
    F_global(1) = 0; F_global(end) = 0;
    
    % Solve the linear system
    u = K_global \ F_global;
end